declare class pTokensNodeProvider {
    private _url;
    /**
     * Create and initialize a pTokensNodeProvider object.
     * @param _url - The URL of a pNetwork node running pNetwork webserver v3.
     */
    constructor(_url: string);
    /**
     * The URL set when creating the object.
     */
    get url(): string;
    /**
     * Send a RPC request to the pNetwork webserver.
     * @param _reqId - The request ID.
     * @param _method - The RPC method to be called.
     * @param _params - Optional parameters that will be passed to the RPC method.
     * @returns A Promise that resolves with the result of the JSON response.
     */
    sendRpcRequest<T>(_reqId: number, _method: string, _params: any[]): Promise<T>;
}

declare enum Status {
    /** Transaction errored  */
    ERROR = -1,
    /** Transaction has been broadcasted */
    DETECTED = 0,
    /** Transaction has been confirmed */
    BROADCASTED = 1
}
declare type InnerTransactionStatus = {
    /** The transaction hash. */
    txHash: string;
    /** The status of the transaction. */
    status: Status;
    /** The chain ID of the blockchain where the transaction took place. */
    chainId: string;
};
declare type TransactionStatus = {
    /** Input transactions. Actually, only one input is supported. */
    inputs: InnerTransactionStatus[];
    /** Output transactions. Actually, only one output is supported. */
    outputs: InnerTransactionStatus[];
};
declare type NativeToXBasisPoints = {
    /** Basis point fees for native-to-host swap. */
    nativeToHost: number;
    /** Basis point fees for native-to-native swap. */
    nativeToNative: number;
};
declare type HostToXBasisPoints = {
    /** Basis point fees for host-to-host swap. */
    hostToHost: number;
    /** Basis point fees for host-to-native swap. */
    hostToNative: number;
};
declare type AssetFees = {
    /** Fees destinated to pay network fees (expressed in token quantity * 1e18). */
    networkFee: number;
    /** Fees destinated to pay network fees (expressed in USD). */
    networkFeeUsd?: number;
    /** Minimum fees destinated to node operators (expressed in token quantity * 1e18). */
    minNodeOperatorFee: number;
    /** Minimum fees destinated to node operators (expressed in USD). */
    minNodeOperatorFeeUsd?: number;
    /** Basis point to calculate node fees destinated to node operators. */
    basisPoints: NativeToXBasisPoints | HostToXBasisPoints;
};
declare type AssetInfo = {
    /** The chain ID of the asset's blockchain. */
    chainId: string;
    /** Boolean indicating if the asset is native in the blockchain. */
    isNative: boolean;
    /** Token smart contract address. */
    tokenAddress: string;
    /** Internal reference which uniquely identifies the asset within the pNetwork. */
    tokenReference: string;
    /** Token's decimals. */
    decimals?: number;
    /** Vault address where tokens shall be transferred for pegging in. */
    vaultAddress?: string;
    /** pNetwork enclave address. */
    identity?: string;
    /** Token-related fees. */
    fees: AssetFees;
};
declare type NativeDepositAddress = {
    nonce: number;
    nativeDepositAddress: string;
    enclavePublicKey: string;
};
declare class pTokensNode {
    private _provider;
    /**
     * Create and initialize a pTokensNode object.
     * @param _provider - A pTokensNodeProvider that will be employed to send requests.
     */
    constructor(_provider: pTokensNodeProvider);
    /** Return the pTokensNodeProvider set when calling constructor. */
    get provider(): pTokensNodeProvider;
    /**
     * Get the transaction status related to a transaction.
     * @param _txHash - The hash of the transaction.
     * @param _originatingChainId - The chain ID where the transaction took place.
     * @returns A Promise that resolves with a TransactionStatus object related to the queried transaction.
     */
    getTransactionStatus(_txHash: string, _originatingChainId: string): Promise<TransactionStatus>;
    /**
     * Get asset info related to those blockchains where pNetwork supports the input symbol.
     * @param _tokenSymbol - The token symbol.
     * @returns An array of AssetInfo objects for those chains where pNetwork supports the specified token.
     */
    getSupportedChainsByAsset(_tokenSymbol: string): Promise<AssetInfo[]>;
    /**
     * Get asset info related to a particular blockchain where pNetwork should support the input symbol.
     * @param _tokenSymbol - The token symbol.
     * @param _chainId - The chain ID of the blockchain.
     * @returns An AssetInfo object for the specified blockchain. Null if the specified blockchain does not support the token.
     */
    getAssetInfoByChainId(_tokenSymbol: string, _chainId: string): Promise<AssetInfo>;
    /**
     * Get a native deposit address where to transfer a UTXO asset for initiating a peg-in.
     * @param _originatingChainId - The chain ID of the source asset.
     * @param _address - The destination asset in the destination chain ID.
     * @param _destinationChainId - The chain ID of the destination asset.
     * @returns A Promise that resolves with a NativeDepositAddress.
     */
    getNativeDepositAddress(_originatingChainId: string, _address: string, _destinationChainId: string): Promise<NativeDepositAddress>;
}

export { AssetFees, AssetInfo, HostToXBasisPoints, InnerTransactionStatus, NativeDepositAddress, NativeToXBasisPoints, Status, TransactionStatus, pTokensNode, pTokensNodeProvider };
