import { http } from 'ptokens-helpers';
import jsonrpc from 'jsonrpc-lite';

var Status;
(function (Status) {
    /** Transaction errored  */
    Status[Status["ERROR"] = -1] = "ERROR";
    /** Transaction has been broadcasted */
    Status[Status["DETECTED"] = 0] = "DETECTED";
    /** Transaction has been confirmed */
    Status[Status["BROADCASTED"] = 1] = "BROADCASTED";
})(Status || (Status = {}));
class pTokensNode {
    /**
     * Create and initialize a pTokensNode object.
     * @param _provider - A pTokensNodeProvider that will be employed to send requests.
     */
    constructor(_provider) {
        this._provider = _provider;
    }
    /** Return the pTokensNodeProvider set when calling constructor. */
    get provider() {
        return this._provider;
    }
    /**
     * Get the transaction status related to a transaction.
     * @param _txHash - The hash of the transaction.
     * @param _originatingChainId - The chain ID where the transaction took place.
     * @returns A Promise that resolves with a TransactionStatus object related to the queried transaction.
     */
    async getTransactionStatus(_txHash, _originatingChainId) {
        return await this._provider.sendRpcRequest(1, 'node_getTransactionStatus', [_txHash, _originatingChainId]);
    }
    /**
     * Get asset info related to those blockchains where pNetwork supports the input symbol.
     * @param _tokenSymbol - The token symbol.
     * @returns An array of AssetInfo objects for those chains where pNetwork supports the specified token.
     */
    async getSupportedChainsByAsset(_tokenSymbol) {
        return await this._provider.sendRpcRequest(1, 'node_getSupportedChainsByAsset', [_tokenSymbol]);
    }
    /**
     * Get asset info related to a particular blockchain where pNetwork should support the input symbol.
     * @param _tokenSymbol - The token symbol.
     * @param _chainId - The chain ID of the blockchain.
     * @returns An AssetInfo object for the specified blockchain. Null if the specified blockchain does not support the token.
     */
    async getAssetInfoByChainId(_tokenSymbol, _chainId) {
        const info = (await this.getSupportedChainsByAsset(_tokenSymbol)).filter((p) => p.chainId == _chainId);
        return info.length ? info.at(0) : null;
    }
    /**
     * Get a native deposit address where to transfer a UTXO asset for initiating a peg-in.
     * @param _originatingChainId - The chain ID of the source asset.
     * @param _address - The destination asset in the destination chain ID.
     * @param _destinationChainId - The chain ID of the destination asset.
     * @returns A Promise that resolves with a NativeDepositAddress.
     */
    async getNativeDepositAddress(_originatingChainId, _address, _destinationChainId) {
        return await this._provider.sendRpcRequest(1, 'node_getNativeDepositAddress', [
            _originatingChainId,
            _address,
            _destinationChainId,
        ]);
    }
}

class pTokensNodeProvider {
    /**
     * Create and initialize a pTokensNodeProvider object.
     * @param _url - The URL of a pNetwork node running pNetwork webserver v3.
     */
    constructor(_url) {
        this._url = _url;
    }
    /**
     * The URL set when creating the object.
     */
    get url() {
        return this._url;
    }
    /**
     * Send a RPC request to the pNetwork webserver.
     * @param _reqId - The request ID.
     * @param _method - The RPC method to be called.
     * @param _params - Optional parameters that will be passed to the RPC method.
     * @returns A Promise that resolves with the result of the JSON response.
     */
    async sendRpcRequest(_reqId, _method, _params) {
        const req = jsonrpc.request(_reqId, _method, _params);
        const resp = await http.fetchJsonByPost(this.url, req);
        if ('error' in resp)
            throw new Error(`JSON RPC error ${resp.error.message}`);
        else if (resp.result)
            return resp.result;
        else
            throw new Error(`Invalid JSON RPC response ${JSON.stringify(resp)}`);
    }
}

export { Status, pTokensNode, pTokensNodeProvider };
